import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import * as React from "react";
import { Layer } from "./Layer";
import { Positioner } from "./Positions";
import { arrowStyles } from "./Tooltip";
import { useFocusElement } from "./Hooks/use-focus-trap";
import { animated } from "react-spring";
import { Sheet } from "./Sheet";
import { useMedia } from "use-media";
import PropTypes from "prop-types";
import { useTheme } from "./Theme/Providers";
import { mergeRefs } from "./Hooks/merge-refs";
//动画效果
const AnimatedLayer = animated(Layer);
/**
 弹出式，像个对话框的，用作菜单列表等。
 */
export const Popover = ({ content, children, placement, closeOnMenuItemClick = true, isOpen: defaultShow = false }) => {
    const theme = useTheme();
    const [show, setShow] = React.useState(defaultShow);
    const child = React.Children.only(children);
    const triggerRef = React.useRef(null);
    const popoverRef = React.useRef(null);
    useFocusElement(popoverRef, show, {
        escapeDeactivates: true,
        clickOutsideDeactivates: true
    });
    const bg = theme.colors.mode === "dark"
        ? theme.colors.background.tint1
        : theme.colors.background.layer;
    function open() {
        setShow(true);
        document.body.addEventListener("click", onBodyClick, false);
        document.body.addEventListener("keydown", onBodyKeyDown, false);
    }
    const unmount = React.useCallback(removeBodyListeners, []);
    //SPA发生路由切换后，会导致先前页面的组件被unmount，假如类似回调函数的钩子还没有处理掉，再次触发调用的，就引起报错 unmounted Error。
    //组件卸载了, "click"的已注册处理函数必须撤销，否则报错。
    React.useEffect(() => {
        return () => unmount();
    }, [unmount]);
    function close() {
        setShow(false);
        removeBodyListeners();
    }
    function removeBodyListeners() {
        document.body.removeEventListener("click", onBodyClick, false);
        document.body.removeEventListener("keydown", onBodyKeyDown, false);
    }
    //看看是否要关闭对话框
    function onBodyClick(e) {
        const trigger = triggerRef.current;
        const popover = popoverRef.current;
        if (!trigger || !popover) {
            return;
        }
        // Don't close if clicking the trigger
        if (trigger === e.target || trigger.contains(e.target)) {
            return;
        }
        // or the popover, sometimes
        if (popover === e.target || popover.contains(e.target)) {
            if (closeOnMenuItemClick) {
                const target = e.target;
                if (target.hasAttribute('[data-trigger-close="true"]') ||
                    target.closest('[data-trigger-close="true"]')) {
                    close();
                    return;
                }
            }
            return;
        }
        close();
    }
    function onBodyKeyDown(e) {
        // close on escape key or enter
        if (e.keyCode === 27) {
            close();
        }
        // close on enter
        if (closeOnMenuItemClick && e.keyCode === 13) {
            close();
        }
    }
    //弹出的对话框必须跟随这个按钮（=Trigger Button）
    function renderTrigger({ ref }) {
        return React.cloneElement(child, {
            onPress: (e) => {
                onTriggerClicked();
                if (child.props.onPress) {
                    child.props.onPress(e);
                }
            },
            ref: mergeRefs(ref, triggerRef),
            role: "button",
            "aria-expanded": show,
            "aria-haspopup": true
        });
    }
    function onTriggerClicked() {
        return show ? close() : open();
    }
    return (_jsx(Positioner, Object.assign({ placement: placement, isOpen: show, target: renderTrigger }, { children: ({ placement, ref, arrowProps, style }, animation) => (_jsxs(AnimatedLayer, Object.assign({ role: "dialog", elevation: "md", ref: mergeRefs(ref, popoverRef), style: Object.assign(Object.assign({}, style), { opacity: animation.opacity }), "data-placement": placement, css: {
                zIndex: theme.zIndices.popover,
                margin: theme.spaces.sm,
                borderRadius: theme.radii.md,
                background: bg
            } }, { children: [_jsx("div", { "data-placement": placement, css: arrowStyles(bg), ref: arrowProps.ref, style: arrowProps.style }, void 0), content] }), void 0)) }), void 0));
};
//运行时刻做检查类型用的。
Popover.propTypes = {
    isOpen: PropTypes.bool,
    children: PropTypes.element,
    content: PropTypes.node,
    closeOnMenuItemClick: PropTypes.bool,
    placement: PropTypes.oneOf([
        "auto-start",
        "auto",
        "auto-end",
        "top-start",
        "top",
        "top-end",
        "right-start",
        "right",
        "right-end",
        "bottom-end",
        "bottom",
        "bottom-start",
        "left-end",
        "left",
        "left-start"
    ])
};
/**
 * Display popover contents in a bottom sheet if
 * on mobile devices. I generally find that this provides a
 * better use experience on smaller screens. 手机电脑自适应的版本。
 */
export const ResponsivePopover = (props) => {
    const theme = useTheme();
    // show popover on iPad+  如果最小宽度 >=md: "629px",
    const showPopover = useMedia({
        minWidth: theme.breakpoints.md
    });
    const [isOpen, setIsOpen] = React.useState(false);
    if (showPopover) {
        //电脑版||大屏的，是这样的：
        return _jsx(Popover, Object.assign({}, props), void 0);
    }
    //手机版本走下面这样的
    return (_jsxs(React.Fragment, { children: [React.cloneElement(React.Children.only(props.children), {
                onPress: () => {
                    setIsOpen(true);
                }
            }), _jsx(Sheet, Object.assign({ position: "bottom", isOpen: isOpen, onRequestClose: () => {
                    setIsOpen(false);
                } }, { children: props.content }), void 0)] }, void 0));
};
ResponsivePopover.propTypes = Object.assign({}, Popover.propTypes);
//# sourceMappingURL=Popover.js.map